@@ -26,6 +26,8 @@ import argparse
 
 from .cifar10.model_ds_cnn import ds_cnn_cifar10
 from .cifar10.model_vgg import vgg_cifar10
+from .utk_face.model_vgg import vgg_utk_face
+from .kws.model_ds_cnn import ds_cnn_kws
 
 
 def main():
@@ -43,6 +45,11 @@ def main():
         "ds_cnn_cifar10",
         help="A Depthwise Separable CIFAR10 model inspired by MobileNet")
     csp.add_parser("vgg_cifar10", help="A VGG-like CIFAR10 model")
+    csp.add_parser("vgg_utk_face", help="A VGG-like UTKFace model")
+    csp.add_parser(
+        "ds_cnn_kws",
+         help="A Depthwise Separable MobileNet-like model for the Keyword" \
+              " Spotting example")
     args = parser.parse_args()
     if args.action == "create":
         # Check extension
@@ -50,6 +57,10 @@ def main():
             model = ds_cnn_cifar10()
         elif args.model == "vgg_cifar10":
             model = vgg_cifar10()
+        elif args.model == "vgg_utk_face":
+            model = vgg_utk_face()
+        elif args.model == "ds_cnn_kws":
+            model = ds_cnn_kws()
         # No need for default behaviour as the command-line parser only accepts
         # valid model types
         if args.save_model is None:
