@@ -105,6 +105,14 @@ def tse_mlp_cse2018(numerical_columns=None,
 
 
 def tse_mlp_cse2018_pretrained():
+    """
+    Helper method to retrieve a `tse_mlp_cse2018` model that was trained on
+    CSE-CIC-IDS-2018 dataset.
+
+    Returns:
+        tf.keras.Model: a Keras Model instance.
+
+    """
     model_name = 'tse_mlp_cse2018_wq2_aq4_iq0.tar.gz'
     model_path = get_file(fname=model_name,
                           origin=BASE_WEIGHT_PATH + model_name,
