@@ -60,7 +60,6 @@ import os
 import warnings
 
 # Tensorflow/Keras Imports
-import tensorflow.keras.utils as keras_utils
 from tensorflow.keras import Model
 from tensorflow.keras.utils import get_file
 from tensorflow.keras.layers import (Input, BatchNormalization, MaxPool2D,
@@ -68,11 +67,10 @@ from tensorflow.keras.layers import (Input, BatchNormalization, MaxPool2D,
                                      Activation)
 
 # cnn2snn Imports
-from cnn2snn import (WeightQuantizer, WeightFloat, QuantizedConv2D,
-                     QuantizedSeparableConv2D, load_quantized_model)
+from cnn2snn import load_quantized_model
 
 # Local utils
-from ..quantization_blocks import _add_activation_layer
+from ..quantization_blocks import conv_block, separable_conv_block
 
 BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/mobilenet/'
 
@@ -126,44 +124,6 @@ def _obtain_input_shape(input_shape, default_size, min_size, include_top):
     return input_shape
 
 
-def _depthwise_block(inputs, filters, name, add_maxpool, weight_quantizer,
-                     activ_quantization):
-
-    x = QuantizedSeparableConv2D(filters=filters,
-                                 kernel_size=(3, 3),
-                                 use_bias=False,
-                                 padding='same',
-                                 name=name,
-                                 quantizer=weight_quantizer)(inputs)
-
-    if add_maxpool:
-        x = MaxPool2D(pool_size=(2, 2), padding='same',
-                      name=name + '_maxpool')(x)
-    x = BatchNormalization(name=name + '_BN')(x)
-    x = _add_activation_layer(x, activ_quantization, name + '_relu')
-
-    return x
-
-
-def _get_weight_quantizer(bitwidth):
-    """Returns a weight quantizer according to the bitwidth.
-
-    # Args
-        bitwidth (int): The quantization bitwidth for weights. If
-            'bitwidth' is zero, the weights are not quantized (float).
-
-    # Returns
-        WeightQuantizer: a weight quantizer object.
-    """
-    if bitwidth == 0:
-        return WeightFloat()
-    elif isinstance(bitwidth, int) and bitwidth > 0:
-        return WeightQuantizer(threshold=1, bitwidth=bitwidth)
-    else:
-        raise ValueError("'bitwidth' argument must be a positive integer, "
-                         "or zero for float weights.")
-
-
 def mobilenet_imagenet(input_shape=None,
                        alpha=1.0,
                        dropout=1e-3,
@@ -251,67 +211,151 @@ def mobilenet_imagenet(input_shape=None,
 
     img_input = Input(shape=input_shape)
 
-    weight_quantizer = _get_weight_quantizer(weight_quantization)
-    weight_quantizer_fl = _get_weight_quantizer(input_weight_quantization)
+    x = conv_block(img_input,
+                   filters=int(32 * alpha),
+                   name='conv_0',
+                   kernel_size=(3, 3),
+                   padding='same',
+                   use_bias=False,
+                   activ_quantization=activ_quantization,
+                   weight_quantization=input_weight_quantization,
+                   add_batchnorm=True)
 
-    x = QuantizedConv2D(filters=int(32 * alpha),
-                        kernel_size=(3, 3),
-                        use_bias=False,
-                        strides=(2, 2),
-                        padding='same',
-                        name='conv_0',
-                        quantizer=weight_quantizer_fl)(img_input)
-    x = BatchNormalization(name='conv_0_BN')(x)
-    x = _add_activation_layer(x, activ_quantization, 'conv_0_relu')
+    x = separable_conv_block(x,
+                             filters=int(64 * alpha),
+                             name='separable_1',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(64 * alpha), 'separable_1', False,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(128 * alpha),
+                             name='separable_2',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             pooling='max',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(128 * alpha), 'separable_2', True,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(128 * alpha),
+                             name='separable_3',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(128 * alpha), 'separable_3', False,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(256 * alpha),
+                             name='separable_4',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             pooling='max',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(256 * alpha), 'separable_4', True,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(256 * alpha),
+                             name='separable_5',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
+    x = separable_conv_block(x,
+                             filters=int(512 * alpha),
+                             name='separable_6',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             pooling='max',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(256 * alpha), 'separable_5', False,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(512 * alpha),
+                             name='separable_7',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(512 * alpha), 'separable_6', True,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(512 * alpha),
+                             name='separable_8',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(512 * alpha), 'separable_7', False,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(512 * alpha),
+                             name='separable_9',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(512 * alpha), 'separable_8', False,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(512 * alpha),
+                             name='separable_10',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(512 * alpha), 'separable_9', False,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(512 * alpha),
+                             name='separable_11',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
-    x = _depthwise_block(x, int(512 * alpha), 'separable_10', False,
-                         weight_quantizer, activ_quantization)
-
-    x = _depthwise_block(x, int(512 * alpha), 'separable_11', False,
-                         weight_quantizer, activ_quantization)
-
-    x = _depthwise_block(x, int(1024 * alpha), 'separable_12', True,
-                         weight_quantizer, activ_quantization)
+    x = separable_conv_block(x,
+                             filters=int(1024 * alpha),
+                             name='separable_12',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             pooling='max',
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
     # Last separable layer with global pooling
-    x = QuantizedSeparableConv2D(filters=int(1024 * alpha),
-                                 kernel_size=(3, 3),
-                                 use_bias=False,
-                                 padding='same',
-                                 name='separable_13',
-                                 quantizer=weight_quantizer)(x)
-
-    if include_top or pooling == 'avg':
-        x = GlobalAvgPool2D(name='separable_13_global_avg')(x)
-    x = BatchNormalization(name='separable_13_BN')(x)
-    x = _add_activation_layer(x, activ_quantization, 'separable_13_relu')
+    layer_pooling = 'global_avg' if include_top or pooling == 'avg' else None
+    x = separable_conv_block(x,
+                             filters=int(1024 * alpha),
+                             name='separable_13',
+                             kernel_size=(3, 3),
+                             padding='same',
+                             pooling=layer_pooling,
+                             use_bias=False,
+                             activ_quantization=activ_quantization,
+                             weight_quantization=weight_quantization,
+                             add_batchnorm=True)
 
     if include_top:
         shape = (1, 1, int(1024 * alpha))
@@ -319,12 +363,15 @@ def mobilenet_imagenet(input_shape=None,
         x = Reshape(shape, name='reshape_1')(x)
         x = Dropout(dropout, name='dropout')(x)
 
-        x = QuantizedSeparableConv2D(filters=classes,
-                                     kernel_size=(3, 3),
-                                     use_bias=False,
-                                     padding='same',
-                                     name='separable_14',
-                                     quantizer=weight_quantizer)(x)
+        x = separable_conv_block(x,
+                                 filters=classes,
+                                 name='separable_14',
+                                 kernel_size=(3, 3),
+                                 padding='same',
+                                 use_bias=False,
+                                 activ_quantization=None,
+                                 weight_quantization=weight_quantization,
+                                 add_batchnorm=False)
         x = Activation('softmax', name='act_softmax')(x)
         x = Reshape((classes,), name='reshape_2')(x)
 
@@ -339,6 +386,14 @@ def mobilenet_imagenet(input_shape=None,
 
 
 def mobilenet_imagenet_pretrained():
+    """
+    Helper method to retrieve a `mobilenet_imagenet` model that was trained on
+    ImageNet dataset.
+
+    Returns:
+        tf.keras.Model: a Keras Model instance.
+
+    """
     model_name = 'mobilenet_imagenet_wq4_aq4.h5'
     model_path = get_file(fname=model_name,
                           origin=BASE_WEIGHT_PATH + model_name,
@@ -347,6 +402,14 @@ def mobilenet_imagenet_pretrained():
 
 
 def mobilenet_cats_vs_dogs_pretrained():
+    """
+    Helper method to retrieve a `mobilenet_imagenet` model that was trained on
+    Cats vs.Dogs dataset.
+
+    Returns:
+        tf.keras.Model: a Keras Model instance.
+
+    """
     model_name = 'mobilenet_cats_vs_dogs_wq4_aq4.h5'
     model_path = get_file(fname=model_name,
                           origin=BASE_WEIGHT_PATH + model_name,
