@@ -66,11 +66,6 @@ _G_MEAN = 116.78
 _B_MEAN = 103.94
 CHANNEL_MEANS = [_R_MEAN, _G_MEAN, _B_MEAN]
 
-# The lower bound for the smallest side of the image for aspect-preserving
-# resizing. For example, if an image is 500 x 1000, it will be resized to
-# _RESIZE_MIN x (_RESIZE_MIN * 2).
-_RESIZE_MIN = 256
-
 
 def process_record_dataset(dataset,
                            is_training,
@@ -653,8 +648,9 @@ def preprocess_image(image_buffer,
         #image = _random_distort_color(image)
     else:
         # For validation, we want to decode, resize, then just crop the middle.
+        resize_min = int(np.round(max(output_height, output_width) * 1.143))
         image = tf.image.decode_jpeg(image_buffer, channels=num_channels)
-        image = _aspect_preserving_resize(image, _RESIZE_MIN)
+        image = _aspect_preserving_resize(image, resize_min)
         image = _central_crop(image, output_height, output_width)
 
     image.set_shape([output_height, output_width, num_channels])
